<?php
/*
 * @component ZoneNationsPay
 * @copyright Copyright (C) 2023 Druskus ZoneNations https://zonenations.com
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\Database\ParameterType;
use Joomla\CMS\MVC\Model\ListModel;

class ZoneNationsPayModelRegionRent extends ListModel
{
    protected $searchInFields = ['region_name'];

    public function __construct($config = array())
    {
        $config['filter_fields'] = $this->searchInFields;
        parent::__construct($config);
    }

    protected function getListQuery()
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true);

        $query->select([
            $db->quoteName('r.id'),
            $db->quoteName('r.region_name'),
            $db->quoteName('r.rental_price'),
            'COUNT(' . $db->quoteName('t.id') . ') AS transaction_count',
            'SUM(' . $db->quoteName('t.amount_rlc') . ') AS total_amount'
        ])
        ->from($db->quoteName('#__zonenationspay_regions', 'r'))
        ->leftJoin($db->quoteName('#__zonenationspay_transactions', 't') . ' ON ' . $db->quoteName('t.region_id') . ' = ' . $db->quoteName('r.id'))
        ->group($db->quoteName('r.id'));

        // Filter search
        $search = $this->getState('filter.search');
        if (!empty($search)) {
            $search = $db->quote('%' . str_replace(' ', '%', $db->escape(trim($search), true) . '%'));
            $query->where($db->quoteName('r.region_name') . ' LIKE ' . $search);
        }

        // Filter by price range
        $minPrice = $this->getState('filter.min_price');
        $maxPrice = $this->getState('filter.max_price');
        if ($minPrice !== null) {
            $query->where($db->quoteName('r.rental_price') . ' >= :minPrice')
                ->bind(':minPrice', $minPrice, ParameterType::FLOAT);
        }
        if ($maxPrice !== null) {
            $query->where($db->quoteName('r.rental_price') . ' <= :maxPrice')
                ->bind(':maxPrice', $maxPrice, ParameterType::FLOAT);
        }

        $query->order($db->quoteName('r.region_name') . ' ASC');

        return $query;
    }

    protected function populateState($ordering = null, $direction = null)
    {
        $app = Factory::getApplication();

        $search = $app->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
        $this->setState('filter.search', preg_replace('/\s+/', ' ', $search));

        $minPrice = $app->getUserStateFromRequest($this->context . '.filter.min_price', 'filter_min_price');
        $this->setState('filter.min_price', $minPrice);

        $maxPrice = $app->getUserStateFromRequest($this->context . '.filter.max_price', 'filter_max_price');
        $this->setState('filter.max_price', $maxPrice);

        parent::populateState('r.region_name', 'ASC');
    }

    public function getRegionRent($id)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__zonenationspay_regions'))
            ->where($db->quoteName('id') . ' = :id')
            ->bind(':id', $id, ParameterType::INTEGER);
        
        $db->setQuery($query);
        return $db->loadObject();
    }

    public function addRegionRent($regionName, $rentalPrice)
    {
        $db = Factory::getDbo();
        $object = new \stdClass();
        $object->region_name = $regionName;
        $object->rental_price = $rentalPrice;

        try {
            $result = $db->insertObject('#__zonenationspay_regions', $object);
            return [
                'success' => true,
                'message' => Text::_('COM_ZONENATIONSPAY_REGION_RENT_ADDED')
            ];
        } catch (Exception $e) {
            return [
                'success' => false,
                'message' => Text::_('COM_ZONENATIONSPAY_REGION_RENT_ADD_ERROR')
            ];
        }
    }

    public function updateRegionRent($id, $regionName, $rentalPrice)
    {
        $db = Factory::getDbo();
        $object = new \stdClass();
        $object->id = $id;
        $object->region_name = $regionName;
        $object->rental_price = $rentalPrice;

        try {
            $result = $db->updateObject('#__zonenationspay_regions', $object, 'id');
            return [
                'success' => true,
                'message' => Text::_('COM_ZONENATIONSPAY_REGION_RENT_UPDATED')
            ];
        } catch (Exception $e) {
            return [
                'success' => false,
                'message' => Text::_('COM_ZONENATIONSPAY_REGION_RENT_UPDATE_ERROR')
            ];
        }
    }

    public function deleteRegionRent($id)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true)
            ->delete($db->quoteName('#__zonenationspay_regions'))
            ->where($db->quoteName('id') . ' = :id')
            ->bind(':id', $id, ParameterType::INTEGER);

        try {
            $db->setQuery($query)->execute();
            return [
                'success' => true,
                'message' => Text::_('COM_ZONENATIONSPAY_REGION_RENT_DELETED')
            ];
        } catch (Exception $e) {
            return [
                'success' => false,
                'message' => Text::_('COM_ZONENATIONSPAY_REGION_RENT_DELETE_ERROR')
            ];
        }
    }

    public function getRegionRentStats($id)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true);

        $query->select([
            'COUNT(' . $db->quoteName('t.id') . ') AS transaction_count',
            'SUM(' . $db->quoteName('t.amount_rlc') . ') AS total_amount',
            'AVG(' . $db->quoteName('t.amount_rlc') . ') AS average_amount',
            'MIN(' . $db->quoteName('t.transactiontime') . ') AS first_transaction',
            'MAX(' . $db->quoteName('t.transactiontime') . ') AS last_transaction'
        ])
        ->from($db->quoteName('#__zonenationspay_transactions', 't'))
        ->where($db->quoteName('t.region_id') . ' = :id')
        ->bind(':id', $id, ParameterType::INTEGER);

        $db->setQuery($query);
        return $db->loadObject();
    }
}