<?php
/*
 * @component ZoneNationsPay
 * @copyright Copyright (C) 2023 Druskus ZoneNations https://zonenations.com
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\Database\ParameterType;

/**
 * ZoneNationsPay Transactions Model
 */
class ZoneNationsPayModelTransactions extends ZoneNationsPayModelZoneNationsPay
{
    protected $searchInFields = ['payer_email', 'payer_firstname', 'payer_lastname', 'opensimid', 'amount_rlc', 'amount_iwc'];

    public function __construct($config = array())
    {
        $config['filter_fields'] = $this->searchInFields;
        parent::__construct($config);
    }

    /**
     * Method to build an SQL query to load the list data.
     *
     * @return      string  An SQL query
     */
    protected function getListQuery()
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true);

        $query->select([
            $db->quoteName('t.id'),
            $db->quoteName('t.payer_email'),
            $db->quoteName('t.amount_rlc'),
            $db->quoteName('t.amount_iwc'),
            $db->quoteName('t.opensimid'),
            $db->quoteName('t.transactiontime'),
            $db->quoteName('t.iwbalance'),
            $db->quoteName('t.mc_fee', 'paypalfee'),
            'CONCAT_WS(" ", ' . $db->quoteName('t.payer_firstname') . ', ' . $db->quoteName('t.payer_lastname') . ') AS name',
            'IFNULL(' . $db->quoteName('c.symbol') . ', "?") AS symbol',
            'IFNULL(' . $db->quoteName('u.username') . ', ' . $db->quote(Text::_('COM_ZONENATIONSPAY_NOTAVAILABLE')) . ') AS joomlausername',
            $db->quoteName('r.region_name'),
            $db->quoteName('r.rental_price')
        ]);

        $query->from($db->quoteName('#__zonenationspay_transactions', 't'))
            ->leftJoin($db->quoteName('#__zonenationspay_currencies', 'c') . ' ON ' . $db->quoteName('t.currencyname') . ' = ' . $db->quoteName('c.currency'))
            ->leftJoin($db->quoteName('#__users', 'u') . ' ON ' . $db->quoteName('t.joomlaid') . ' = ' . $db->quoteName('u.id'))
            ->leftJoin($db->quoteName('#__zonenationspay_regions', 'r') . ' ON ' . $db->quoteName('t.region_id') . ' = ' . $db->quoteName('r.id'));

        // Filter search
        $search = $this->getState('filter.search');
        if (!empty($search)) {
            $search = $db->quote('%' . str_replace(' ', '%', $db->escape(trim($search), true) . '%'));
            $query->where('(' . implode(' OR ', array_map(function($field) use ($db, $search) {
                return $db->quoteName('t.' . $field) . ' LIKE ' . $search;
            }, $this->searchInFields)) . ')');
        }

        // Filter payment status
        $paymentstatus = $this->getState('filter.paymentstatus');
        if (!empty($paymentstatus)) {
            $query->where($db->quoteName('t.payment_type') . ' = :paymentstatus')
                ->bind(':paymentstatus', $paymentstatus);
        }

        // Filter daterange
        $daterange = $this->getState('filter.daterange');
        if (!empty($daterange)) {
            $query->where($db->quoteName('t.transactiontime') . ' >= DATE_SUB(NOW(), INTERVAL :daterange DAY)')
                ->bind(':daterange', $daterange, ParameterType::INTEGER);
        }

        $query->order($db->quoteName('t.transactiontime') . ' DESC');

        return $query;
    }

    /**
     * Method to auto-populate the model state.
     *
     * Note. Calling getState in this method will result in recursion.
     *
     * @param   string  $ordering   An optional ordering field.
     * @param   string  $direction  An optional direction (asc|desc).
     *
     * @return  void
     *
     * @since   1.6
     */
    protected function populateState($ordering = null, $direction = null)
    {
        $app = Factory::getApplication();

        // Load the filter state.
        $search = $app->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
        $this->setState('filter.search', preg_replace('/\s+/', ' ', $search));

        // Filter (dropdown) payment status
        $paymentstatus = $app->getUserStateFromRequest($this->context . '.filter.paymentstatus', 'filter_paymentstatus', '', 'string');
        $this->setState('filter.paymentstatus', $paymentstatus);

        // Filter (dropdown) daterange
        $daterange = $app->getUserStateFromRequest($this->context . '.filter.daterange', 'filter_daterange', '', 'string');
        $this->setState('filter.daterange', $daterange);

        // List state information.
        parent::populateState($ordering, $direction);
    }

    /**
     * Method to get transaction details including region rent information.
     *
     * @param   int  $transactionId  The ID of the transaction.
     *
     * @return  object|false  The transaction details or false if not found.
     */
    public function getTransactionDetails($transactionId)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true);

        $query->select([
            't.*',
            'r.region_name',
            'r.rental_price'
        ])
        ->from($db->quoteName('#__zonenationspay_transactions', 't'))
        ->leftJoin($db->quoteName('#__zonenationspay_regions', 'r') . ' ON ' . $db->quoteName('t.region_id') . ' = ' . $db->quoteName('r.id'))
        ->where($db->quoteName('t.id') . ' = :transactionId')
        ->bind(':transactionId', $transactionId, ParameterType::INTEGER);

        $db->setQuery($query);

        return $db->loadObject();
    }
}