<?php
/**
 * @component ZoneNationsPay
 * @copyright Copyright (C) 2023 Druskus ZoneNations https://www.zonenations.com
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') or die();
use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Log\Log;

require_once(JPATH_COMPONENT . '/models/zonenationspay.php');

class ZonenationspayModelNotify extends ZonenationspayModelZoneNationsPay
{
    public $filename = "notify.php";
    public $view = "notify";
    public $postvals = [];
    public $imodel = null;
    protected $log2file = false;
    protected $failure2file = false;
    protected $paypaldebug = true;
    protected $curlopt_ssl_verifypeer;
    protected $curlopt_ssl_verifyhost;

    public function __construct($config = array())
    {
        parent::__construct($config);
        
        $this->curlopt_ssl_verifypeer = $this->params->get('paypal_nossl', 0) ? false : 1;
        $this->curlopt_ssl_verifyhost = $this->params->get('paypal_nossl', 0) ? false : 2;

        $logsetting = $this->params->get('log2file', 0);
        $this->log2file = ($logsetting & 2) == 2;
        $this->failure2file = ($logsetting & 1) == 1;
    }

    public function validate()
    {
        $app = Factory::getApplication();
        $this->postvals = $app->input->post->getArray();

        $req = 'cmd=_notify-validate';
        foreach ($this->postvals as $key => $value) {
            $req .= "&$key=" . urlencode(stripslashes($value));
        }

        $ch = curl_init();
        curl_setopt_array($ch, [
            CURLOPT_URL => $this->params->get('paypal_action'),
            CURLOPT_POST => 1,
            CURLOPT_POSTFIELDS => $req,
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_SSL_VERIFYPEER => $this->curlopt_ssl_verifypeer,
            CURLOPT_SSL_VERIFYHOST => $this->curlopt_ssl_verifyhost,
            CURLOPT_HTTPHEADER => ['Host: ' . $this->params->get('paypalhost', 'www.paypal.com')]
        ]);
        $res = curl_exec($ch);
        curl_close($ch);
        return $res;
    }

    public function handleResponse($res, $debug = false)
    {
        $paymentStatus = $debug ? "Completed" : ($this->postvals['payment_status'] ?? '');
        $business = $debug ? "paypaltest@zonenations.com" : ($this->postvals['business'] ?? '');
        $txnId = $debug ? "97417185GE1381330" : ($this->postvals['txn_id'] ?? '');
        if ($paymentStatus !== "Completed") {
            $this->logError("Payment status is not Completed: $paymentStatus");
            return false;
        }
        if ($business !== $this->params->get('paypal_account')) {
            $this->logError("Business account mismatch: $business");
            return false;
        }

        $db = $this->getDbo();
        $query = $db->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__zonenationspay_transactions'))
            ->where($db->quoteName('txn_id') . ' = ' . $db->quote($txnId));
        $db->setQuery($query);
        if ($db->loadResult()) {
            $this->logError("Transaction ID already exists: $txnId");
            return false;
        }

        $currency = $this->params->get('currency');
        $mcCurrency = $debug ? "EUR" : ($this->postvals['mc_currency'] ?? '');
        if ($currency !== $mcCurrency) {
            $this->logError("Currency mismatch: $mcCurrency");
            return false;
        }

        $payment = $debug ? "10.00" : ($this->postvals['mc_gross'] ?? 0);
        $minPay = $this->params->get('minbuy', 0);
        $maxPay = $this->params->get('maxbuy', 0);

        if ($minPay > 0 && $payment < $minPay) {
            $this->logWarning("Payment below minimum: $payment < $minPay");
        } elseif ($maxPay > 0 && $payment > $maxPay) {
            $this->logWarning("Payment above maximum: $payment > $maxPay");
        }

        $userdata = $debug ? ["0e88c38e-9e09-4c8d-8ee4-5b22accfc2b4", "62"] : explode(":", $this->postvals['custom'] ?? '');
        if (count($userdata) !== 2) {
            $this->logError("Invalid custom value from PayPal");
            return false;
        }

        // Process payment
        $rate = $this->params->get('currencyratebuy', 1);
        $fee = $this->params->get('transactionfee', 0);
        $feeType = $this->params->get('transactionfeetype', 'fixed');
        
        $basePayment = $this->calculateBasePayment($payment, $fee, $feeType);
        $iwCurrency = floor($basePayment * $rate);

        // Transfer money in-world
        $transferResult = $this->transferMoneyInWorld($userdata[0], $iwCurrency);
        if (!$transferResult) {
            $this->logError("Failed to transfer money in-world");
            return false;
        }

        // Record transaction
        $this->recordTransaction($txnId, $userdata, $payment, $iwCurrency, $fee, $feeType);

        // Check for region rent payment
        $this->handleRegionRentPayment($userdata[1], $iwCurrency);

        return true;
    }

    protected function calculateBasePayment($payment, $fee, $feeType)
    {
        if ($fee > 0) {
            return $feeType === "percent" ? $payment / (100 + $fee) * 100 : $payment - $fee;
        }
        return $payment;
    }

    protected function transferMoneyInWorld($receiverId, $amount)
    {
        $banker = $this->getMoneySetting("bankerUID");
        $parameter = [
            'senderID' => $banker,
            'receiverID' => $receiverId,
            'amount' => $amount,
            'description' => $this->params->get('transactiontext', 'ZoneNationsPay Transfer')
        ];
        return $this->imodel->subcall("TransferMoney", $parameter);
    }

    protected function recordTransaction($txnId, $userdata, $payment, $iwCurrency, $fee, $feeType)
    {
        $db = $this->getDbo();
        $columns = [
            'txn_id', 'opensimid', 'joomlaid', 'amount_rlc', 'amount_iwc', 
            'fee', 'feetype', 'transactiontime', 'mode'
        ];
        $values = [
            $db->quote($txnId),
            $db->quote($userdata[0]),
            (int)$userdata[1],
            (float)$payment,
            (int)$iwCurrency,
            $db->quote($fee),
            $db->quote($feeType),
            $db->quote(Factory::getDate()->toSql()),
            $db->quote($this->params->get('paypal_sandbox', 0) ? 'sandbox' : 'live')
        ];

        $query = $db->getQuery(true)
            ->insert($db->quoteName('#__zonenationspay_transactions'))
            ->columns($db->quoteName($columns))
            ->values(implode(',', $values));
        $db->setQuery($query);
        $db->execute();

        $this->logInfo("Transaction recorded: $txnId");
    }

    protected function handleRegionRentPayment($userId, $amount)
    {
        $db = $this->getDbo();
        $query = $db->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__zonenationspay_regionrents'))
            ->where($db->quoteName('user_id') . ' = ' . (int)$userId)
            ->where($db->quoteName('status') . ' = ' . $db->quote('unpaid'))
            ->order($db->quoteName('due_date') . ' ASC');

        $db->setQuery($query);
        $unpaidRents = $db->loadObjectList();

        foreach ($unpaidRents as $rent) {
            if ($amount >= $rent->amount) {
                $this->payRegionRent($rent->id, $amount);
                $amount -= $rent->amount;
            } else {
                break;
            }
        }
    }

    protected function payRegionRent($rentId, $amount)
    {
        $db = $this->getDbo();
        $query = $db->getQuery(true)
            ->update($db->quoteName('#__zonenationspay_regionrents'))
            ->set([
                $db->quoteName('status') . ' = ' . $db->quote('paid'),
                $db->quoteName('paid_date') . ' = ' . $db->quote(Factory::getDate()->toSql()),
                $db->quoteName('paid_amount') . ' = ' . (float)$amount
            ])
            ->where($db->quoteName('id') . ' = ' . (int)$rentId);

        $db->setQuery($query);
        $db->execute();

        $this->logInfo("Region rent paid: $rentId");
    }

    public function setMoneyModel($imodel)
    {
        $this->imodel = $imodel;
    }

    protected function logError($message)
    {
        Log::add($message, Log::ERROR, 'zonenationspay');
        if ($this->failure2file) {
            $this->debugzeile(['error' => $message], "ERROR");
        }
        if ($this->params->get('notifyerror', 0)) {
            $this->sendnotify(1, 3, "ERROR: $message\n\n" . var_export($this->postvals, true));
        }
    }

    protected function logWarning($message)
    {
        Log::add($message, Log::WARNING, 'zonenationspay');
        if ($this->failure2file) {
            $this->simpledebugzeile("WARNING: $message");
        }
        if ($this->params->get('notifywarning', 0)) {
            $this->sendnotify(2, 2, "WARNING: $message");
        }
    }

    protected function logInfo($message)
    {
        Log::add($message, Log::INFO, 'zonenationspay');
        if ($this->log2file) {
            $this->simpledebugzeile($message);
        }
        if ($this->params->get('notifytransaction', 0)) {
            $this->sendnotify(3, 1, $message);
        }
    }
}
