<?php
/**
 * @component ZoneNationsPay
 * @copyright Copyright (C) 2023 Druskus ZoneNations https://www.zonenations.com
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') or die();

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Date\Date;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;

require_once(JPATH_COMPONENT . '/models/zonenationspay.php');

class ZonenationspayModelPayout extends ZonenationspayModelZoneNationsPay
{
    public $filename = "payout.php";
    public $view = "payout";

    public function savePayoutRequest()
    {
        $app = Factory::getApplication();
        $user = Factory::getUser();
        $opensimid = $this->getUUID($user->id);
        
        if (!$opensimid) {
            return $this->returnError(null, __LINE__);
        }

        $requestedPayout = $app->input->getInt('zonenationspay_amount', 0);
        $receiverPayPal = $app->input->getString('paypaluser', '');
        $payoutrequest = $app->input->getFloat('payoutvalue', 0);
        $amount_iwc = $app->input->getInt('zonenationspay_amount', 0);
        $currency_rlc = $app->input->getString('currency_code', '');
        $xchangerate = $app->input->getFloat('zonenationspay_xchangerate', 0);
        $transactionfee = $app->input->getFloat('transactionfee', 0);
        $transactionfeetype = $app->input->getString('transactionfeetype', '');
        $requestip = $app->input->server->getString('REMOTE_ADDR');
        $payoutValue = $this->getPayoutValue($requestedPayout);

        // Validation checks
        if ($requestedPayout == 0) {
            return $this->returnError(null, __LINE__);
        }
        if (!$this->sufficientBalance($opensimid, $requestedPayout)) {
            return $this->returnError(1);
        }
        if (!filter_var($receiverPayPal, FILTER_VALIDATE_EMAIL)) {
            return $this->returnError(2);
        }
        if ($payoutValue < 0) {
            return $this->returnError(3);
        }
        if ($payoutValue != $payoutrequest) {
            return $this->returnError(4);
        }

        $db = $this->getDbo();
        $date = new Date();
        
        $columns = [
            'joomlaid', 'opensimid', 'amount_iwc', 'amount_rlc', 'currency_rlc', 
            'xchangerate', 'transactionfee', 'transactionfeetype', 'paypalaccount', 
            'requesttime', 'requestip', 'status'
        ];
        $values = [
            $user->id, $db->quote($opensimid), $db->quote($amount_iwc), 
            $payoutValue, $db->quote($currency_rlc), $db->quote($xchangerate), 
            $db->quote($transactionfee), $db->quote($transactionfeetype), 
            $db->quote($receiverPayPal), $db->quote($date->toSql()), 
            $db->quote($requestip), 0
        ];

        $query = $db->getQuery(true)
            ->insert($db->quoteName('#__zonenationspay_payoutrequests'))
            ->columns($db->quoteName($columns))
            ->values(implode(',', $values));
        $db->setQuery($query);
        $result = $db->execute();

        if ($result) {
            if ($this->params->get('notifypayout', 0) == 1) {
                $message = sprintf(
                    "Name: %s\nUsername: %s\nJoomlaID: %d\nOpenSimUID: %s\n%s: %s\n%s: %d",
                    $user->name, $user->username, $user->id, $opensimid,
                    $currency_rlc, $payoutValue, $this->getMoneySetting('name'), $amount_iwc
                );
                $this->sendnotify(4, 4, $message);
            }

            // Check if this payout affects any region rent
            $this->checkAndUpdateRegionRent($user->id, $amount_iwc);
        }
        return $result;
    }

    private function sufficientBalance($opensimid, $request)
    {
        $balance = $this->getBalance($opensimid);
        return $balance >= $request;
    }

    public function returnError($errnum = 0, $line = null)
    {
        $retval = [
            'success' => false,
            'error' => $errnum,
            'errormsg' => Text::_('COM_ZONENATIONSPAY_UNKNOWNERROR')
        ];
        switch ($errnum) {
            case 1:
                $retval['errormsg'] = Text::sprintf('COM_ZONENATIONSPAY_BALANCE_SELLERROR', $this->params->get('inworld_currency_name', 'IWC'));
                break;
            case 2:
                $retval['errormsg'] = Text::_('COM_ZONENATIONSPAY_PAYPALERROR');
                break;
            case 3:
                $retval['errormsg'] = Text::sprintf('COM_ZONENATIONSPAY_ERROR_NEGATIVE', $this->params->get('inworld_currency_name', 'IWC'));
                break;
            case 4:
                $retval['errormsg'] = Text::_('COM_ZONENATIONSPAY_UNKNOWNERROR') . " (!=)";
                break;
            default:
                $retval['errormsg'] .= $line ? " (Line: $line)" : '';
                break;
        }
        return $retval;
    }

    public function getPayoutValue($requestvalue)
    {
        $rlc = $requestvalue / $this->params->get('currencyratesell', 1);
        $fee = 0;

        if ($this->params->get('hasfee', false)) {
            $feetype = $this->params->get('transactionfeetype', '');
            $feeAmount = $this->params->get('transactionfee', 0);

            switch ($feetype) {
                case "percent":
                    $fee = $rlc * ($feeAmount / 100);
                    break;
                case "amount":
                    $fee = $feeAmount;
                    break;
            }
        }

        $payoutvalue = $rlc - $fee;
        return round($payoutvalue, 2);
    }

    private function checkAndUpdateRegionRent($userId, $amount)
    {
        $db = $this->getDbo();
        $query = $db->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__zonenationspay_regionrents'))
            ->where($db->quoteName('user_id') . ' = ' . (int)$userId)
            ->where($db->quoteName('status') . ' = ' . $db->quote('unpaid'))
            ->order($db->quoteName('due_date') . ' ASC');

        $db->setQuery($query);
        $unpaidRents = $db->loadObjectList();

        foreach ($unpaidRents as $rent) {
            if ($amount >= $rent->amount) {
                $this->payRegionRent($rent->id, $amount);
                $amount -= $rent->amount;
            } else {
                break;
            }
        }
    }

    private function payRegionRent($rentId, $amount)
    {
        $db = $this->getDbo();
        $query = $db->getQuery(true)
            ->update($db->quoteName('#__zonenationspay_regionrents'))
            ->set($db->quoteName('status') . ' = ' . $db->quote('paid'))
            ->set($db->quoteName('paid_date') . ' = ' . $db->quote(Factory::getDate()->toSql()))
            ->set($db->quoteName('paid_amount') . ' = ' . (float)$amount)
            ->where($db->quoteName('id') . ' = ' . (int)$rentId);

        $db->setQuery($query);
        return $db->execute();
    }
}
