<?php
/**
 * @component ZoneNationsPay
 * @copyright Copyright (C) 2023 Druskus ZoneNations https://www.zonenations.com
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') or die();

use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;

require_once(JPATH_COMPONENT . '/models/zonenationspay.php');

class ZonenationspayModelPaypal extends ZonenationspayModelZoneNationsPay
{
    public $filename = "paypal.php";
    public $view = "paypal";

    public function __construct($config = array())
    {
        parent::__construct($config);
    }

    // ... [previous methods remain unchanged] ...
    public function processPayPalPayment($ipnData)
    {
        // Verifica IPN con PayPal
        if (!$this->verifyPayPalIPN($ipnData)) {
            throw new Exception('Invalid IPN verification');
        }

        $db = $this->getDbo();
        $date = Factory::getDate();
        $app = Factory::getApplication();

        // Inizia una transazione del database
        $db->transactionStart();

        try {
            // Estrai le informazioni necessarie dall'IPN
            $joomlaId = $this->getUserIdFromCustom($ipnData['custom']);
            $amountRlc = $ipnData['mc_gross'];
            $amountIwc = $this->convertToIwc($amountRlc);
            $payerEmail = $ipnData['payer_email'];
            $txnId = $ipnData['txn_id'];

            // Inserisci la transazione nel database
            $columns = array('joomlaid', 'amount_rlc', 'amount_iwc', 'transactiontime', 'payer_email', 'txn_id', 'status');
            $values = array(
                (int)$joomlaId,
                (float)$amountRlc,
                (float)$amountIwc,
                $db->quote($date->toSql()),
                $db->quote($payerEmail),
                $db->quote($txnId),
                $db->quote('completed')
            );

            $query = $db->getQuery(true)
                ->insert($db->quoteName('#__zonenationspay_transactions'))
                ->columns($db->quoteName($columns))
                ->values(implode(',', $values));

            $db->setQuery($query);
            $db->execute();

            // Aggiorna il saldo dell'utente
            $this->updateUserBalance($joomlaId, $amountIwc);

            // Se c'è un affitto regione associato, aggiornalo
            if (isset($ipnData['item_number']) && strpos($ipnData['item_number'], 'regionrent_') === 0) {
                $rentId = substr($ipnData['item_number'], 11);
                $this->updateRegionRentStatus($rentId, 'paid');
            }

            // Commit della transazione
            $db->transactionCommit();

            // Invia una notifica all'utente
            $this->sendPaymentNotification($joomlaId, $amountRlc, $amountIwc);

            return true;
        } catch (Exception $e) {
            // Rollback in caso di errore
            $db->transactionRollback();
            $app->enqueueMessage($e->getMessage(), 'error');
            return false;
        }
    }

    protected function verifyPayPalIPN($ipnData)
    {
        // Determina se stiamo usando l'ambiente sandbox o quello di produzione
        $paypalUrl = $this->params->get('paypal_sandbox', 0) 
            ? 'https://ipnpb.sandbox.paypal.com/cgi-bin/webscr' 
            : 'https://ipnpb.paypal.com/cgi-bin/webscr';

        // Inizializza la richiesta cURL
        $ch = curl_init();

        // Imposta le opzioni cURL
        curl_setopt_array($ch, [
            CURLOPT_URL => $paypalUrl,
            CURLOPT_POST => true,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_HTTPHEADER => [
                'User-Agent: PHP-IPN-Verification-Script',
                'Connection: Close',
            ],
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_SSL_VERIFYPEER => true,
            CURLOPT_SSL_VERIFYHOST => 2,
            CURLOPT_FORBID_REUSE => true,
            CURLOPT_CONNECTTIMEOUT => 30,
            CURLOPT_TIMEOUT => 60,
        ]);

        // Prepara i dati da inviare
        $encodedData = 'cmd=_notify-validate';
        foreach ($ipnData as $key => $value) {
            $encodedData .= "&$key=" . urlencode(stripslashes($value));
        }

        curl_setopt($ch, CURLOPT_POSTFIELDS, $encodedData);

        // Esegui la richiesta e ottieni la risposta
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        // Chiudi la connessione cURL
        curl_close($ch);

        // Verifica la risposta
        if ($httpCode !== 200) {
            $this->logError("IPN verification failed. HTTP code: $httpCode");
            return false;
        }

        if ($response === 'VERIFIED') {
            // Verifica aggiuntiva dei dati IPN
            return $this->validateIPNData($ipnData);
        } else {
            $this->logError("IPN verification failed. Response: $response");
            return false;
        }
    }

    protected function validateIPNData($ipnData)
    {
        // Verifica che lo stato del pagamento sia completato
        if ($ipnData['payment_status'] !== 'Completed') {
            $this->logError("Payment status is not Completed: " . $ipnData['payment_status']);
            return false;
        }

        // Verifica che la valuta sia corretta
        $expectedCurrency = $this->params->get('paypal_currency', 'USD');
        if ($ipnData['mc_currency'] !== $expectedCurrency) {
            $this->logError("Unexpected currency: " . $ipnData['mc_currency']);
            return false;
        }

        // Verifica che l'email del destinatario del pagamento sia corretta
        $expectedReceiver = $this->params->get('paypal_email');
        if ($ipnData['receiver_email'] !== $expectedReceiver) {
            $this->logError("Unexpected receiver email: " . $ipnData['receiver_email']);
            return false;
        }

        // Verifica che la transazione non sia già stata processata
        if ($this->isTransactionProcessed($ipnData['txn_id'])) {
            $this->logError("Transaction already processed: " . $ipnData['txn_id']);
            return false;
        }

        return true;
    }

    protected function isTransactionProcessed($txnId)
    {
        $db = $this->getDbo();
        $query = $db->getQuery(true)
            ->select('COUNT(*)')
            ->from($db->quoteName('#__zonenationspay_transactions'))
            ->where($db->quoteName('txn_id') . ' = ' . $db->quote($txnId));

        $db->setQuery($query);
        return (bool)$db->loadResult();
    }

    protected function logError($message)
    {
        $logger = Factory::getApplication()->getLogger();
        $logger->error('ZoneNationsPay PayPal IPN: ' . $message);
    }

    // ... [other methods remain unchanged] ...
}